
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *   Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 *
 * $Id: dram.ld,v 1.8 1998/04/11 23:42:21 steve Exp $
 *
 * $Log: dram.ld,v $
 * Revision 1.8  1998/04/11 23:42:21  steve
 *  Correct end of heap symbol.
 *
 * Revision 1.7  1998/04/03 01:35:12  steve
 *  Add assertion support.
 *
 * Revision 1.6  1997/09/04 07:37:54  steve
 *  Get rid of excessive _ in xTOR_TABLE symbols.
 *
 * Revision 1.5  1997/09/03 23:45:02  steve
 *  Add support for dynamic loading, including the
 *  commands to make symbol tables, read and link
 *  modules, and download fondled binaries.
 *
 * Revision 1.4  1997/08/25 17:43:02  steve
 *  Allow memory layout be set by link models.
 *
 * Revision 1.3  1996/10/28 03:45:05  steve
 *  License notice.
 *
 * Revision 1.2  1996/10/11 00:23:18  steve
 *  Rearrange the memory map to reflect the theory of
 *  operation for the ISE board.
 *
 * Revision 1.1  1996/10/08 03:28:48  steve
 *  Add the ISE bootstrap prom.
 *
 */

/*
 * The memory map for the linked result places everything but the IBR in
 * dram. The image is loaded onto the board by a BOOTPROM that resides in
 * flash memory and uses high DRAM for its own stack and data. The IBR is
 * linked into its usual place, but the boot prom doesn't actually write
 * it, it uses the numbers to start the process.
 */

MEMORY
{
	/* Put DRAM in region 2_3. */
	dram  : org = 0xa0000000, len = 4M

	  /* FLASH memory is fixed in hardware to be 512K bytes,
	     and we want it at the end so we can put the IMI there,
	     especially the initial boot record. */
	flash : org = 0xff000000 - 512K, len = 512K
	IBR   : org = 0xfeffff30,  len = 208
}

	  /* The ISE board has a couple little registers for controlling
	     the few non-PCI devices on the local bus. */

__ISE_DRAM    = 0xa0000000;
__ISE_FLASH   = 0xfef80000;
__ISE_STATUS  = 0xb0000000;
__ISE_CONTROL = 0xb0000000;


ENTRY(start)
STARTUP(locore.o)

SECTIONS
{
	  /* Executable code, constants, and the constructor table
	     go in the beginning of the memory. */

	.text : {
		__index_space__ = .;
		. += 1024;
		*(.text)
		. = ALIGN(4);
		__CTOR_TABLE__ = .;
		*(.ctors)
		LONG(0);
		__DTOR_TABLE__ = .;
		*(.dtors)
		LONG(0);
	} > dram 


	  /* Initialized data goes into the DRAM, right afer the
	     .text section. Note that I take special care that the
	     _etext and _bdata symbols are adjacent. This causes
	     the startup code to NOT copy the .data section on
	     when the program starts. */

	.data : {
		. = ALIGN(16); _etext = .; _bdata = .;
		*(.data)
		*(COMMON)
		_edata = .;
	} > dram


	  /* The .bss section is uninitialized data. I need to link
	     to it, but it doesn't get loaded. */

	.bss (NOLOAD) : {
		*(.bss)
		. = ALIGN(16);
		_end = .;
	}  >dram

	_end_heap = __ISE_DRAM + 4M;


	  /* The IBR is special and magical. It must be in a very
	     specific place, so placing it there is the job of the
	     linker, that's me. */

	ibr : {
		*(ibr)
	} > IBR


}
ibr_checksum = -(start + prcb);
