#ifndef __chips_LED_H
#define __chips_LED_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: LED.h,v 1.3 1997/04/14 05:43:10 demo Exp $"
/*
 * $Log: LED.h,v $
 * Revision 1.3  1997/04/14 05:43:10  demo
 *  Update init to support rev 3 ISE boards.
 *
 * Revision 1.2  1996/10/28 03:45:12  steve
 *  License notice.
 *
 * Revision 1.1  1996/07/23 23:57:20  steve
 *  Add the LED template class.
 *
 */

/*
 * This is a generic device driver for LED registers. These registers
 * are extremely simple devices, usually latches, that match a bit to
 * an LED. The template parameter is typically an integral data type,
 * and the number of bits in the type is the number of LEDs available.
 *
 * The implementations herin are not interrupt or thread-safe because
 * of the read-modify-write cycle for the bits_ member. However, if
 * you feel clever, you can create a type that has atomic |= and &=
 * operators, and then the LED class would be interrupt and thread safe.
 */

template <class RT> class LED {

    public:
	// The init value is the initial value of the register. The
	// class assumes the register is not readable, so this is
	// needed to know how to write the whole register to change
	// just a single bit.
      explicit LED(volatile RT*base, RT init = 0)
      : base_(base), bits_(init)
      { *base_ = bits_; }

      void set(unsigned idx)
      { bits_ |= (1<<idx); *base_ = bits_; }

      void clr(unsigned idx)
      { bits_ &= ~(1<<idx); *base_ = bits_; }

      void assign(RT value)
      { bits_ = value; *base_ = bits_; }

    private:
      volatile RT*const base_;
      RT bits_;

    private:
      LED(const LED&);
      LED& operator= (const LED&);
};

#endif
