#ifndef __chips_M29F040_H
#define __chips_M29F040_H
/*
 * Copyright (c) 1997 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *  ---
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 */
#ident "$Id: M29F040.h,v 1.1 1998/01/01 20:44:15 steve Exp $"

# include  <stddef.h>

/*
 * This class implements th flash operations of the M29F040 Flash
 * memory from SGS Thompson. Compatible devices should work (the
 * manufacturer and device codes are not checked) as is.
 *
 * All the addresses used by the various methods are byte offsets into
 * the part. This is true even for the items that refer to blocks--the
 * address is the offset of the first byte in the block. The only
 * exception is the constructor, which is given the base address of
 * the part in memory space.
 */
class M29F040 {

    public:
	// Create a class with the address in memory of the part. Once
	// created. This is the absolute address of the part. All the
	// other addresses in this class are offsets within the part,
	// and use this base address for calculations.

      explicit M29F040(volatile void*base);

	// Read part identification bytes.

      unsigned char manufacturer() const;
      unsigned char device_code() const;

      enum { BLOCK_SIZE = 64*1024, CHIP_SIZE = 512*1024 };

	// Return true if the block is protected. This driver does not
	// know how to unprotect blocks (microprocessor bussus don't
	// do the needed waveforms) so a protected block cannot be
	// erased.

      bool block_protected(unsigned long addr) const;

	// Erase the block at the given address. Erasure is
	// potentially slow (measured in seconds) so the method will
	// yield the thread until the chip reports erase complete.

      void erase_block(unsigned long addr);

	// This method programs a single byte as the specified
	// location within the part. Writing typically means
	// programming zeros to the right bits, so the block must be
	// erased first.

      void program_bytes(unsigned long addr,
			 const unsigned char*byte, size_t cnt);


    private:
      volatile unsigned char*base_;

    private: // not implemented
      M29F040(const M29F040&);
      M29F040& operator= (const M29F040&);
};

/*
 * $Log: M29F040.h,v $
 * Revision 1.1  1998/01/01 20:44:15  steve
 *  Add flash driver
 *
 */
#endif
