#ifndef __chips_SC2692_H
#define __chips_SC2692_H
/*
 * Copyright (c) 1997 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: SC2692.h,v 1.4 1997/11/25 17:59:08 demo Exp $"
/*
 * $Log: SC2692.h,v $
 * Revision 1.4  1997/11/25 17:59:08  demo
 *  Support setting Parity.
 *
 * Revision 1.3  1997/08/02 04:28:59  steve
 *  Some standard constants.
 *
 * Revision 1.2  1997/04/14 05:42:06  demo
 *  Rearrange for speed and constructibility.
 *
 * Revision 1.1  1997/04/03 23:24:28  steve
 *  Add SC2692 device class.
 *
 */

/*
 * The SC2692 class represents the Philips Semiconductor SSC2692 and
 * compatible dual UART chips. This includes the 2681 DUART.
 */
class SC2692 {

    private:
      enum RREGS_ { SRA = 1, RxFIFOA = 3, IPCR = 4, ISR = 5, SRB = 9,
		    IPR = 13 };
      enum WREGS_ { MRA = 0, CSRA = 1, CRA = 2, TxFIFOA = 3, ACR = 4,
		    IMR = 5, OPCR = 13 };
		    
      void write_reg_(WREGS_ idx, unsigned char val)
      { * ((volatile unsigned char*)base_ + idx * reg_width_) = val; }

      unsigned char read_reg_(RREGS_ idx) const
      { return * ((volatile unsigned char*)base_ + idx * reg_width_); }

    public:
	// Construct an instance with the base address and the width
	// of the register map. The base is the address of the first
	// register in the part, and the width is the reg_width is the
	// number of bytes between registers (for example, address
	// lines are shifted.)
      explicit SC2692(volatile void*base, unsigned reg_width = 1)
      { init(base, reg_width); }

      SC2692() : base_(0), reg_width_(0) { }
      ~SC2692();

      void init(volatile void*base, unsigned reg_width = 1);

	// Set the baud rate. This takes one of the standard baud rate
	// numbers, in the form of an integer.
      void set_baud_a(unsigned);
      void set_baud_b(unsigned);

      enum PARITY { NONE, MARK, SPACE, EVEN, ODD };
      void set_parity_a(PARITY);
      void set_parity_b(PARITY);

	// Transmit and receive character values. The number of bits
	// actually transmitted depends on the UART configuration.
      void transmit_a(char value);
      void transmit_b(char value);

      char receive_a();
      char receive_b();

      bool RxReady_a() const { return read_reg_(SRA) & 1; }
      bool RxReady_b() const { return read_reg_(SRB) & 1; }

    public:
	// These methods control the chips interrupt signal behavior.
      void int_RxReady_a(bool);

	// Turn local loopback mode on or off, depending on the flag.
      void diag_local_loop(bool flag);

    private:
      volatile void* base_;
      unsigned reg_width_;

    private: // Not implemented
      SC2692(const SC2692&);
      SC2692& operator= (const SC2692&);
};


#endif
