#ifndef __chips_bus_stdio_H
#define __chips_bus_stdio_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: bus_stdio.h,v 1.2 1997/08/02 04:28:59 steve Exp $"

# include  <StdioDevice.h>
class BUS;

/*
 * Create one of these to arrange for stdio access to the BUS. The
 * caller must arrange for the creation of an appropriate BUS object,
 * and select a base string, but once that is done the fopen call will
 * be able to find the device type and open it for you.
 *
 * The device names created are of the form str:<n> where str is the
 * string passed to the constructor and <n> is an unsigned
 * integer. The open allocates (with new) a StdioDevice with a
 * BUS::Channel and uses the channel <n>.
 *
 * An example:
 *
 *    BUS bus;
 *    I960RP i960rp;
 *    bus.open_device(&i960rp);
 *
 *    ChannelDeviceType channels ("channel", &bus);
 *
 *    FILE*out = fopen("channel:13", "w");
 *       [...]
 *    fclose(out);
 *
 * This class requires the libstdio librar be linked to the final
 * application as well.
 */

class ChannelDevice;

class ChannelDeviceType  : public StdioDeviceType {

    public:
      ChannelDeviceType(const char*str, BUS*b)
      : StdioDeviceType(str), bus_(b), list_(0)
      { }

      ~ChannelDeviceType();

    private:
      StdioDevice*open(const char*name);

      void close(StdioDevice*dev);

    private:
      BUS*bus_;
      ChannelDevice*list_;
};



#endif
/*
 * $Log: bus_stdio.h,v $
 * Revision 1.2  1997/08/02 04:28:59  steve
 *  Some standard constants.
 *
 * Revision 1.1  1997/02/21 21:22:14  steve
 *  Incorporate stdio interfacce to BUS cnannels.
 */
