#ifndef __ctype_H
#define __ctype_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: ctype.h,v 1.2 1997/04/25 18:45:50 steve Exp $"
/*
 * $Log: ctype.h,v $
 * Revision 1.2  1997/04/25 18:45:50  steve
 *  More ISO C support.
 *
 * Revision 1.1  1996/10/28 02:16:33  steve
 *  Add support for ctype routines.
 *
 */

/*
 * This header implements the character handling functions defined by
 * the ANSO/ISO Standard for Programming Languages - C ANSI/ISO 9899-1990
 * Section 7.3 Character Handling.
 */

#if defined(__cplusplus)
extern "C" {
#endif

#ifndef EOF
# define EOF (-1)
#endif

/*
 * The implementations of these routines presume ASCII. Some clever
 * bit operations take place here to do the jobs expected. They assume
 * that 0 <= c <= UCHAR_MAX.
 */

extern inline int isdigit(int c) { return ((unsigned)(c - '0')) < 10; }
extern int isspace(int c);
extern inline int isupper(int c) { return ((unsigned)(c - 'A')) < 26; }
extern inline int islower(int c) { return ((unsigned)(c - 'a')) < 26; }
extern inline int iscntrl(int c) { return (c < ' ') || (c == 0x7f); }

extern inline int isalpha(int c) { return islower(c) || isupper(c); }
extern inline int isalnum(int c) { return islower(c) || isupper(c) || isdigit(c); }

extern inline int isxdigit(int c)
      {
	    if (isdigit(c)) return 1;
	    if ((unsigned)(c - 'A') < 6) return 1;
	    if ((unsigned)(c - 'a') < 6) return 1;
	    return 0;
      }

extern inline int toupper(int c)
      {
	    if (islower(c)) return c & ~0x20;
	    else return c;
      }

extern inline int tolower(int c)
      {
	    if (isupper(c)) return c | 0x20;
	    else return c;
      }

#if defined(__cplusplus)
};
#endif

#endif
