#ifndef __i960_fault_asm_H
#define __i960_fault_asm_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: fault_asm.h,v 1.4 1997/02/17 21:06:31 steve Exp $"
/*
 * $Log: fault_asm.h,v $
 * Revision 1.4  1997/02/17 21:06:31  steve
 *  access fault type does not modify record.
 *
 * Revision 1.3  1996/11/07 01:53:02  steve
 *  portaable access to the fault type.
 *
 * Revision 1.2  1996/10/28 03:45:15  steve
 *  License notice.
 *
 * Revision 1.1  1996/09/19 04:43:51  steve
 *  Add the i960_ise target, change the target
 *  configuration headers to allow sharing by
 *  targets, fix up the make clean rule.
 *
 * Revision 1.1  1996/08/10 22:06:52  steve
 *  Add generic fault handling code.
 *
 */

/*
 * This is the fault record for the i960 processor.
 */

struct FAULT_RECORD {
      unsigned long resume_info[4];

      unsigned long pad;
      unsigned long override_data[3];

      unsigned long fault_data[3];
      unsigned char osubtype, opad, otype, oflags;

      unsigned long pc;
      unsigned long ac;
      unsigned char subtype, fpad, ftype, fflags;
      const void* ip;
};

/*
 * The i960 has 16 fault numbers, although many are reserved. Also,
 * not all i960 types generate all defined faults.
 */
enum {
      FAULT_OVERRIDE   =  0,
      FAULT_TRACE      =  1,
      FAULT_OPERATION  =  2,
      FAULT_ARITHMETIC =  3,
      FAULT_CONSTRAINT =  5,
      FAULT_PROTECTION =  7,
      FAULT_TYPE       = 10,

      FAULT_COUNT = 16
};

/*
 * Use this instead of peeking directly into the structure, because
 * different architectures have different structures and different
 * fault meanings.
 */
inline unsigned fault_type(const FAULT_RECORD*frp)
{ return frp->ftype; }

#endif
