#ifndef __mc68322_RGP_H
#define __mc68322_RGP_H

/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: RGP.h,v 1.5 1996/10/28 03:45:20 steve Exp $"
/*
 * $Log: RGP.h,v $
 * Revision 1.5  1996/10/28 03:45:20  steve
 *  License notice.
 *
 * Revision 1.4  1995/10/13 18:44:01  steve
 *  RGP takes pointers to const.
 *
 * Revision 1.3  1995/09/11 01:07:17  steve
 *  Ask for a bit address in blt2ub_XD
 *  General method for writing longs into program.
 *
 * Revision 1.2  1995/09/08 05:59:09  steve
 *  Encapsulate RGP programs
 *
 * Revision 1.1  1995/09/07 21:48:33  steve
 *  Add the RGP class.
 *
 */

/*
 * This class drives the MC68322 RISC Graphics Processor. It executes
 * programs assembled by other means. See the RGProgram class below.
 */

# include  <ucr/ISync.h>
# include  <ucr/heap.h>

class RGProgram;

class RGP {

    public:
      explicit RGP(unsigned char level, unsigned long base =0xfff000);
      ~RGP();

        // XX For now, just execute the single display list
	// synchronously.
      void go(RGProgram&);

    private:
      ISync sync_;

    private:
      struct REGS_ {
	    void*volatile start_;
	    void*volatile diag_;
      };

      struct INTR_ {
	    volatile unsigned short enable_flags;
	    volatile unsigned short intr_flags;
	    volatile unsigned short level;
      };

      REGS_* const regs_;
      INTR_* const intr_;

      static void isr(RGP*, unsigned);
};

/*
 * this class makes the building of programs more convenient. It is
 * also the ideal way to deliver programs to the RGP processor.
 */
class RGProgram {

      friend class RGP;

    public:
      explicit RGProgram(unsigned words, HEAP_SPACE*);
      ~RGProgram();

    public:
      void st_word(unsigned short value)
	{ program_[fill_++] = value; }

      void st_long(unsigned long value)
	{ st_word(value>>16); st_word(value&0xffff); }

        // These are the program steps that go into a program. They
	// are described in chapter 13 of the mc68322 User's Manual.
    public:
      void stop() { st_word(0x0000); }

        // 5 words
      void blt2f_D(void*dest, unsigned short width, unsigned short height);

        // 9 words
      void blt2ub_XD(unsigned long dest,
		     unsigned short dwidth, unsigned short dheight,
		     const void*source, unsigned short swarp,
		     unsigned short zoom);

    private:
      HEAP_SPACE*const heap_;

      unsigned short*program_;
      const unsigned limit_;
      unsigned fill_;

    private: // Not implemented
      RGProgram(const RGProgram&);
      RGProgram& operator= (const RGProgram&);
};

#endif
