#ifndef __ucr_ILock_H
#define __ucr_ILock_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: ILock.h,v 1.5 1996/10/28 03:45:21 steve Exp $"
/*
 * $Log: ILock.h,v $
 * Revision 1.5  1996/10/28 03:45:21  steve
 *  License notice.
 *
 * Revision 1.4  1996/08/06 03:46:07  steve
 *  Add a simple test flag.
 *
 * Revision 1.3  1996/07/30 04:50:27  steve
 *  Thorough documentation.
 *
 * Revision 1.2  1996/07/30 03:13:35  steve
 *  Fix broken test_and_set function.
 *
 * Revision 1.1  1996/07/24 00:00:07  steve
 *  Add the ILock class
 *
 */

/*
 * This class is like a semaphore, but is strictly binary. It can be
 * either true or false. If true, a get() will succeed immediately,
 * leaving the lock held. Subsequent get()s will block until the lock
 * is released. The lock is released with the put() method. Only a
 * held lock can be put().
 *
 * The wait() method will wait for the lock to be released, but will
 * not grab it. This method is useful, for example, for waiting for a
 * device to complete an operation.
 *
 * The put() method makes no sense if the lock is not held, and
 * calling it will cause an assertion failure.
 *
 * Since get() and wait() may block, they may not be called by
 * interrupt handlers.
 */

# include  <ucr/ISync.h>

class ILock  : private ISync {

    public:
      explicit ILock(bool flag =true);
      ~ILock();

    public:

	// Get the lock. Block the thread if the lock is already got,
	// and remain blocked until the put is called.
      void get();

	// Release the lock. Awaken exactly one blocked thread if
	// there are threads waiting for the lock.
      void put();

	// Wait for the lock to become available, but do not grab it
	// afterwards.
      void wait();

      bool got() const { return !flag_; }

    private:
      volatile bool flag_;
};


#endif
