
#
# Copyright (c) 1995 Picture Elements Inc.
#	Stephen Williams
#
#    This source code is free software; you can redistribute it
#    and/or modify it in source code form under the terms of the GNU
#    Library General Public License as published by the Free Software
#    Foundation; either version 2 of the License, or (at your option)
#    any later version. In order to redistribute the software in
#    binary form, you will need a Picture Elements Binary Software
#    License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Library General Public License for more details.
#
#    You should have received a copy of the GNU Library General Public
#    License along with this program; if not, write to the Free
#    Software Foundation, Inc.,
#    59 Temple Place - Suite 330
#    Boston, MA 02111-1307, USA
#
#    You should also have recieved a copy of the Picture Elements
#    Binary Software License offer along with the source. This offer
#    allows you to obtain the right to redistribute the software in
#    binary (compiled) form. If you have not received it, contact
#    Picture Elements, Inc.,
#    777 Panoramic Way
#    Berkeley, CA 94704.
#
#ident "$Id: rules.mk,v 1.21 1998/01/21 04:34:18 steve Exp $"
#
# $Log: rules.mk,v $
# Revision 1.21  1998/01/21 04:34:18  steve
#  Add .rbt rule to the core rules.
#
# Revision 1.20  1997/09/04 01:21:22  steve
#  Rule for generating a symbol table.
#
# Revision 1.19  1997/09/02 19:33:59  steve
#  Change build to use i960-ucr compilers.
#
# Revision 1.18  1997/08/02 04:29:53  steve
#  typo in C++ from target file rule.
#
# Revision 1.17  1997/08/01 05:51:39  steve
#  Get rid of silly internal variables.
#
# Revision 1.16  1997/04/19 06:19:00  steve
#  Nasty bug making .o from target specific .c
#
# Revision 1.15  1997/02/05 05:25:03  steve
#  Add support for xact images.
#
# Revision 1.14  1996/11/12 04:17:39  steve
#  Add processed assembly rules.
#
# Revision 1.13  1996/10/28 03:45:33  steve
#  License notice.
#
# Revision 1.12  1996/10/08 05:22:24  steve
#  Support different target types
#
# Revision 1.11  1996/09/21 16:39:27  steve
#  Select correct way to make images.
#
# Revision 1.10  1996/08/24 20:48:17  steve
#  All warnings, please.
#
# Revision 1.9  1996/08/22 23:13:46  steve
#  No default TARGET.
#
# Revision 1.8  1996/07/31 04:35:05  steve
#  Make .bin from .coff by stripping.
#
# Revision 1.7  1996/05/27 22:41:55  steve
#  Combine dependency and .o generation rules.
#
# Revision 1.6  1996/05/24 01:46:37  steve
#  Initial port to i960_cyclone
#  Seperate some files in the library to seperate files
#  for making smaller binaries, use uCR_ prefix for private
#  symbols, move assembly files around for compile-time
#  portability, change __start to ucr_start and give it
#  new behavior, and simplify the ucr_assert implementation.
#
# Revision 1.5  1995/12/08 05:32:10  steve
#  Include debugging information.
#
# Revision 1.4  1995/10/18 06:11:34  steve
#  Make a target specific include directory.
#
# Revision 1.3  1995/09/07 18:32:43  steve
#  Fix broken library handling
#
# Revision 1.2  1995/09/07 18:11:21  steve
#  Add more generalized makefiles that properly
#  handle libraries all over the place.
#
# Revision 1.1  1995/09/02 18:27:15  steve
#  Initial import of UCR core to cvs.
#
#

ifndef TARGET
TARGET := notarget
endif

OBJD := $(TARGET)

include $(UCR)/make/$(TARGET).mk

ifndef COFF2SCOF
COFF2SCOF := $(UCR)/make/coff2scof.sh
endif

ifndef COFF2SREC
COFF2SREC := $(UCR)/make/coff2srec.sh
endif

INCL := -I$(UCR)/include -I$(UCR)/$(OBJD)/include
LIBR := -L$(UCR)/$(OBJD)/lib
CFLAGS   := -Wall -W -O2 -g
CXXFLAGS := $(CFLAGS)
AFLAGS	 :=

# --
# Parsing Xact rawbit files into binary images is so common that I put
# it in the core ucr and include this make rule. Making a .o from a
# .img is given below.

%.img: %.rbt $(UCR)/util/bin/xact2img
	$(UCR)/util/bin/xact2img $< $@

# --
#  These rules are similar to the following, but allow me to use target
#  specific source files. This is how I conditionally compile parts
#  that are very different. These rules come first because I prefer generating
#  an object from a target specific file if one exists.

$(OBJD)/%.o: %-$(TARGET).s
	$(AS) $(AFLAGS) -o $@ $<

$(OBJD)/%.o $(OBJD)/%.d: %-$(TARGET).cc
	@[ -d $(OBJD) ] || mkdir $(OBJD)
	$(CXX) $(CXX_TFLAGS) $(INCL) $(CXXFLAGS) -D$(TARGET)=1 -MD -c $< -o $(@:%.d=%.o)
	@sed -e 's;^$*-$(TARGET).o;$(@:%.d=%.o) $(@:%.o=%.d);'  $*-$(TARGET).d > $(@:%.o=%.d)
	@rm $*-$(TARGET).d

$(OBJD)/%.o $(OBJD)/%.d: %-$(TARGET).c
	@[ -d $(OBJD) ] || mkdir $(OBJD)
	$(CC) $(CC_TFLAGS) $(INCL) $(CXXFLAGS) -D$(TARGET)=1 -MD -c $< -o $(@:%.d=%.o)
	@sed -e 's;^$*-$(TARGET).o;$(@:%.d=%.o) $(@:%.o=%.d);'  $*-$(TARGET).d > $(@:%.o=%.d)
	@rm $*-$(TARGET).d

$(OBJD)/%.o $(OBJD)/%.d: %-$(TARGET).S
	@[ -d $(OBJD) ] || mkdir $(OBJD)
	$(CC) $(CC_TFLAGS) $(INCL) -D$(TARGET)=1 -MD -c $< -o $(@:%.d=%.o)
	@sed -e 's;^$*-$(TARGET).o;$(@:%.d=%.o) $(@:%.o=%.d);'  $*-$(TARGET).d > $(@:%.o=%.d)
	@rm $*-$(TARGET).d

# --
#  Make an object and dependency file for the given source, C or C++.
#  Use the -MD switch of gcc so that we do the two at the same time,
#  and store the results in the OBJD directory.

$(OBJD)/%.o: %.s
	$(AS) $(AFLAGS) -o $@ $<

$(OBJD)/%.o $(OBJD)/%.d: %.c
	@[ -d $(OBJD) ] || mkdir $(OBJD)
	$(CC) $(CC_TFLAGS) $(INCL) $(CFLAGS) -D$(TARGET)=1 -MD -c $< -o $(@:%.d=%.o)
	@sed -e 's;^$*.o;$(@:%.d=%.o) $(@:%.o=%.d);'  $*.d > $(@:%.o=%.d)
	@rm $*.d


$(OBJD)/%.o $(OBJD)/%.d: %.cc
	@[ -d $(OBJD) ] || mkdir $(OBJD)
	$(CXX) $(CXX_TFLAGS) $(INCL) $(CXXFLAGS) -D$(TARGET)=1 -MD -c $< -o $(@:%.d=%.o)
	@sed -e 's;^$*.o;$(@:%.d=%.o) $(@:%.o=%.d);'  $*.d > $(@:%.o=%.d)
	@rm $*.d

$(OBJD)/%.o $(OBJD)/%.d: %.S
	@[ -d $(OBJD) ] || mkdir $(OBJD)
	$(CC) $(CC_TFLAGS) $(INCL) -D$(TARGET)=1 -MD -c $< -o $(@:%.d=%.o)
	@sed -e 's;^$*.o;$(@:%.d=%.o) $(@:%.o=%.d);'  $*.d > $(@:%.o=%.d)
	@rm $*.d

# --
# This rule makes a linkable object file from a file full of bytes.
$(OBJD)/%.o $(OBJD)/%.d: %.img
	@[ -d $(OBJD) ] || mkdir $(OBJD)
	$(UCR)/make/img2obj.sh $< $(patsubst %.img,$(OBJD)/%.o,$<)
	@echo >  $(patsubst %.img,$(OBJD)/%.d,$<)


# --
# Use the gensyms program to generate a symbol table from a .syms
# file. Make automatically figures out that it can make a .o from the
# .s which is a product of this rule.
%.s: %.syms $(UCR)/util/bin/gensyms
	$(UCR)/util/bin/gensyms < $< > $@

# --
#  Sometimes I need to look at the assembler for a given C or C++ file.
#  This rule generates the .s file from the .c or .cc file. Magically,
#  make will keep it up to date, and use the .s to make .o files as well.

%.s: %.c
	$(CC) $(CC_TFLAGS) $(INCL) $(CFLAGS) -D$(TARGET)=1 -S $<

%.s: %.cc
	$(CC) $(CXX_TFLAGS) $(INCL) $(CXXFLAGS) -D$(TARGET)=1 -S $<

