/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 */
#ident "$Id: coff2bin.cc,v 1.2 1996/10/28 03:45:35 steve Exp $"
/*
 * $Log: coff2bin.cc,v $
 * Revision 1.2  1996/10/28 03:45:35  steve
 *  License notice.
 *
 * Revision 1.1  1995/09/02 18:27:16  steve
 *  Initial import of UCR core to cvs.
 *
 *
 */


# include  "m68k-coff.h"
# include  <sys/types.h>
# include  <sys/stat.h>
# include  <sys/mman.h>
# include  <fcntl.h>
# include  <assert.h>
# include  <iostream.h>
# include  <iomanip.h>

main(int argc, char*argv[])
{
      if (argc != 3) {
	    cerr << "Usage: " << argv[0] << " <coff> <output>" <<
	      endl;
	    return 1;
      }

      int rc;
      int fd = open(argv[1], O_RDONLY, 0);
      if (fd == -1) {
	    perror(argv[1]);
	    return 1;
      }

      struct stat stat;
      rc = fstat(fd, &stat);
      assert( rc != -1 );

      void*file = mmap(0, stat.st_size, PROT_READ, MAP_SHARED, fd, 0);
      assert( file != (void*)-1 );

      external_filehdr*hdr = (external_filehdr*)file;


      external_scnhdr*section = (external_scnhdr*)((char*)(hdr+1) +
						   hdr->f_opthdr[1]);

      external_scnhdr *text = 0, *data = 0, *bss = 0;

      for (unsigned idx = 0 ;  idx < hdr->f_nscns[1] ;  idx += 1) {
	    if (strcmp(".text", section[idx].s_name) == 0)
	      text = section + idx;
	    else if (strcmp(".data", section[idx].s_name) == 0)
	      data = section + idx;
	    else if (strcmp(".bss", section[idx].s_name) == 0)
	      bss = section + idx;
	    else
	      ;
      }

      if (!text) {
	    cerr << argv[1] << ": Missing .text!" << endl;
	    return 1;
      }

      if (!data) {
	    cerr << argv[1] << ": Missing .data!" << endl;
	    return 1;
      }

      if (!bss) {
	    cerr << argv[1] << ": Missing .bss!" << endl;
	    return 1;
      }

      unsigned long text_bytes = *(unsigned long*)(text->s_scnptr);
      unsigned long text_count = *(unsigned long*)(text->s_size);

      unsigned long data_bytes = *(unsigned long*)(data->s_scnptr);
      unsigned long data_count = *(unsigned long*)(data->s_size);

      unsigned long bss_count = *(unsigned long*)(bss->s_size);

      cout << "ROM: "
	<< setw(7) << text_count << " (text) + " << setw(7) <<
	data_count << " (data) = " << setw(7) <<
	(text_count + data_count) << endl;
     


      int ofd = open(argv[2], O_WRONLY|O_TRUNC|O_CREAT, 0666);
      if (ofd == -1) {
	    perror(argv[2]);
	    return 1;
      }

      rc = write(ofd, ((char*)file)+text_bytes, text_count);
      assert( rc == text_count );

      rc = write(ofd, ((char*)file)+data_bytes, data_count);
      assert( rc == data_count );

      cout << "RAM: "
	<< setw(7) << data_count << " (data) + " << setw(7) <<
	bss_count << " ( bss) = " <<
	setw(7) << (data_count + bss_count) << endl;

      return 0;
}
