/*
 * Copyright (c) 1997 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: BUScwrite.cc,v 1.3 1998/05/26 16:16:33 steve Exp $"
/*
 * $Log: BUScwrite.cc,v $
 * Revision 1.3  1998/05/26 16:16:33  steve
 *  New channel protocol
 *
 * Revision 1.2  1998/01/01 20:45:16  steve
 *  ISync has const methods.
 *
 * Revision 1.1  1997/06/16 05:14:44  steve
 *  New channel protocol design and implementation.
 *
 */

# include  <chips/BUS.h>
# include  <assert.h>

bool BUS::Channel::wait_for_waddr_(const volatile void*o)
{
      Channel*obj = (Channel*)o;
      if (obj->woff_ < obj->wsize_)
	    return true;

      obj->master_->check_channel_status_(obj);
      return obj->woff_ < obj->wsize_;
}

void BUS::Channel::write(const void*buffer, size_t count)
{
      assert( master_ );
      char*buf = (char*)buffer;

      while (count > 0) {
	    master_->dispatch_sync_.sleep(&wait_for_waddr_, this);

	    size_t trans = count;
	    if (trans + woff_ > wsize_)
		  trans = wsize_ - woff_;

	    master_->dev_->bus_dma_write(waddr_+woff_, buf, trans);

	    count -= trans;
	    woff_ += trans;
	    buf   += trans;

	    if (woff_ == wsize_)
		  master_->consume_in_buffer_(this);
     }

      if (woff_)
	    master_->consume_in_buffer_(this);
}
