/*
 * Copyright (c) 1998 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *  ---
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 */
#ident "$Id: ZR36050.cc,v 1.1 1998/01/19 02:34:17 steve Exp $"

# include  <chips/ZR36050.h>

size_t ZR36050::compressed_size() const
{
      size_t result = 4; // SOI and EOI markers

      unsigned char markers = get_register(6);

      if (markers & 0x80) { // APPx marker
	    result += 2;
	    result += get_register(0x382) << 8;
	    result += get_register(0x383);
      }

      if (markers & 0x40) { // COM marker
	    result += 2;
	    result += get_register(0x3c2) << 8;
	    result += get_register(0x3c3);
      }

      if (markers & 0x10) { // DQT marker
	    result += 2;
	    result += get_register(0xce) << 8;
	    result += get_register(0xcf);
      }

      if (markers & 0x08) { // DHT marker
	    result += 2;
	    result += get_register(0x1d6) << 8;
	    result += get_register(0x1d7);
      }

	// This chip produces only one frame, so count just one SOF.
      result += 2;
      result += get_register(0x42) << 8;
      result += get_register(0x43);

	// XXXX FIXME! Presume exactly one scan.
      {
	    result += 2;
	    result += get_register(0x7c) << 8;
	    result += get_register(0x7d);
      }

	// Count the code volume produced.
      result += get_register(0x16) << 21;
      result += get_register(0x17) << 13;
      result += get_register(0x18) << 5;
      result += get_register(0x19) >> 3;

      return result;
}


/*
 * $Log: ZR36050.cc,v $
 * Revision 1.1  1998/01/19 02:34:17  steve
 *  Add new chip drivers
 *
 */
