
/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: ParallelPort_byte.cc,v 1.2 1996/10/28 03:45:52 steve Exp $"
/*
 * $Log: ParallelPort_byte.cc,v $
 * Revision 1.2  1996/10/28 03:45:52  steve
 *  License notice.
 *
 * Revision 1.1  1995/11/02 06:57:17  steve
 *  Split up the ParallelPort source,
 *  Support nibble mode id strings.
 *
 */

# include  <mc68322/ParallelPort.h>
# include  "ParallelPort_priv.h"
# include  <ucr/ucr.h>
# include  <assert.h>


void ParallelPort::enter_byte_mode_()
{
      port_intr_->enable_flags = LOW_nSelectIn|HIGH_nAutoFd;
      port_isr_ = &port_isr_byte_;
      reverse_transfer_ = &reverse_byte_;

        // Wakeup a thread that may be writing to the buffer.
      out_sync_.wakeup();
}

void ParallelPort::port_isr_byte_()
{
        // If the host terminates the transfer, complete the handshake
	// to go back to compatability mode.
      if (0 == (port_regs_->lines & PIN_nSelectIn))
	terminate_ieee1284_();
}

void ParallelPort::reverse_byte_()
{
      unsigned char byte = *out_buffer_;

        // Event  7: Host lowers nAutoFd to ask for data
      while (port_regs_->lines & PIN_nAutoFd)
	;


        // Event 15: I put the byte onto the bus, and...
      port_regs_->ppcr = 0x0040|PPCR_OUT_ENABLE|PPCR_FILTER;
      port_regs_->data = byte;

        // Event 9: Lower nAck to tell host it's ready.
      port_regs_->lines |= PIN_nAck2;


      while (0 == (port_regs_->lines & PIN_nAutoFd))
	/* Wait for Event 10: host raises nAutoFd. */;

      out_remaining_ -= 1;
      out_buffer_ += 1;

        // Event 13: Send back my status.
      if (out_remaining_ > 0)
	port_regs_->lines = PIN_Select|PIN_nAck2;
      else
	port_regs_->lines = PIN_Select
	                   |PIN_nFault
			   |PIN_PError
			   |PIN_nAck2;

}
