
/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: ParallelPort_ecp.cc,v 1.2 1996/10/28 03:45:52 steve Exp $"
/*
 * $Log: ParallelPort_ecp.cc,v $
 * Revision 1.2  1996/10/28 03:45:52  steve
 *  License notice.
 *
 * Revision 1.1  1995/11/02 06:57:17  steve
 *  Split up the ParallelPort source,
 *  Support nibble mode id strings.
 *
 */

# include  <mc68322/ParallelPort.h>
# include  "ParallelPort_priv.h"
# include  <ucr/ucr.h>
# include  <assert.h>

/*
 * Entering ECP mode requires a little more wiggling to synchronize
 * with the host the initial values of the lines. I also presume that
 * the host wants forward mode. If otherwise, it will initiate a
 * switch.
 */
void ParallelPort::enter_ecp_mode_()
{

      while (port_regs_->lines & PIN_nAutoFd)
	/* Wait for event 30: host lowers nAutoFd */;

        // Event 31: Peripheral accepts entry into ECP mode.
      port_regs_->lines |= PIN_PError;

        // ECP mode is initially forward. Let hardware handle it.
      reverse_transfer_ = 0;
      if (rle_flag_)
	port_regs_->ppcr = PPCR_ECPRLE_MODE|PPCR_FILTER;
      else
	port_regs_->ppcr = PPCR_ECP_MODE|PPCR_FILTER;
      in_flag_ = true;

        // Watch for end of ECP mode, or direction changes.
      port_isr_ = &port_isr_ecp_forward_;
      port_intr_->enable_flags = CMD_RECEIVED|LOW_nSelectIn|LOW_nInit;
      in_sync_.wakeup();
}

/*
 * This isr method detects transitions when in ECP FORWARD mode.
 */
void ParallelPort::port_isr_ecp_forward_()
{
      unsigned short lines = port_regs_->lines;

        // Event 39: host requests reverse transfer.
      if (0 == (lines & (PIN_nAutoFd|PIN_nInit))) {
	    port_regs_->lines = PIN_Select;
	    reverse_transfer_ = &reverse_ecp_;
	    port_isr_ = &port_isr_ecp_reverse_;
	    port_intr_->enable_flags = LOW_nSelectIn|HIGH_nInit;
	    in_flag_ = false;
	    out_sync_.wakeup();
      }

      
      // XXX Here is where I should handle a received command.

        // If the host terminates the transfer, complete the handshake
	// to go back to compatability mode. This is the start of
	// Event 22.
      if (0 == (port_regs_->lines & PIN_nSelectIn)) {
	    in_flag_ = false;
	    terminate_ieee1284_();
      }

}

void ParallelPort::port_isr_ecp_reverse_()
{
        // Event 47: Host requests switch back to forward mode
      if (PIN_nInit == (port_regs_->lines & PIN_nInit)) {

	      // Event 48: Respond to switch request by disabling the
	      // bus and turning on the nAck line.
	    port_regs_->ppcr = 0|PPCR_FILTER;
	    port_regs_->lines = PIN_Select;

	    nap_for_host_();
	    reverse_transfer_ = 0;
	    port_isr_ = &port_isr_ecp_forward_;
	    port_intr_->enable_flags = CMD_RECEIVED|LOW_nSelectIn|LOW_nInit;

	      // Event 49: After setup delay, turn off reverse mode
	      // and resume forward transfer.
	    port_regs_->lines = PIN_Select | PIN_PError;
	    if (rle_flag_)
	      port_regs_->ppcr = PPCR_ECPRLE_MODE|PPCR_FILTER;
	    else
	      port_regs_->ppcr = PPCR_ECP_MODE|PPCR_FILTER;

	    in_flag_ = true;
	    in_sync_.wakeup();
      }

        // If the host terminates the transfer, complete the handshake
	// to go back to compatability mode. This is the start of
	// Event 22.
      if (0 == (port_regs_->lines & PIN_nSelectIn))
	terminate_ieee1284_();
}


/*
 * When in ECP reverse mode, I can write bytes to the host. Write as
 * many as possible, but watch out for change of state. The interrupt
 * handler will tell me about such things by changing the
 * reverse_transfer_ member.
 */
void ParallelPort::reverse_ecp_()
{
      port_regs_->ppcr = PPCR_OUT_ENABLE|PPCR_FILTER;
      bool got_run = false;

      while ((out_remaining_ > 0) && (reverse_transfer_ == &reverse_ecp_)) {

	    unsigned short nCMD = PIN_Select|PIN_Busy2;
	    unsigned char byte = *out_buffer_;

	      // This is the number of bytes I handle in this
	      // handshake. Normally one, but if I get off a run
	      // length, then it could be different.
	    size_t increment = 1;

	      // Look for a run that I can compress. If I find one,
	      // find out how far I can go, then replace the current
	      // byte with the run length. The next time around, then,
	      // the byte to be repeated will be transmitted.
	    if (rle_flag_
		&& (out_remaining_ >= 3)
		&& (out_buffer_[1] == out_buffer_[0])
		&& (out_buffer_[2] == out_buffer_[0]))

	      if (got_run) {
		      // Watch out! If I already found a run and I'm
		      // outputting the byte that follows the run
		      // length, do NOT start another run length.
		    got_run = false;

	      } else {
		    unsigned char count = 3;
		    while ( (count <= 0x7f)
			   && (count < out_remaining_)
			   && (out_buffer_[count] == *out_buffer_) )
		      count += 1;

		      // Count is now the index to the first different
		      // byte.

		    nCMD = PIN_Select;
		    byte = count - 1;
		    increment = count-1;
		    got_run = true;
	      }

	      // -- //
	      // After all that, the following is really all there is
	      // to transmitting a byte. The better part of the above
	      // mess is detecting run lengths, and handling back-to-
	      // back runs.

	      // Event 42: Drive the byte going to the host
	    port_regs_->lines = nCMD;
	    port_regs_->data = byte;

	      // Event 43: Strobe the byte with the nAck line
	    port_regs_->lines = nCMD|PIN_nAck2;

	      // Wait for Event 44: Host acks the byte.
	    for (;;) {
		  unsigned short pins = port_regs_->lines;
		  if (pins & PIN_nAutoFd) break;
		  if (! (pins & PIN_nSelectIn)) break;
	    }

	    if (! (port_regs_->lines & PIN_nSelectIn)) break;

	      // Event 45: deassert the nAck to finish handshake
	    port_regs_->lines = PIN_Select|PIN_Busy2;

	    out_remaining_ -= increment;
	    out_buffer_ += increment;

	      // Wait for Event 46: Host finished handshake
	    while (port_regs_->lines & PIN_nAutoFd)
	      /* busy wait */;
      }
}
