
/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: TimerModule.cc,v 1.5 1996/10/28 03:45:57 steve Exp $"
/*
 * $Log: TimerModule.cc,v $
 * Revision 1.5  1996/10/28 03:45:57  steve
 *  License notice.
 *
 * Revision 1.4  1995/10/03 03:34:32  steve
 *  Get time calculations correct.
 *
 * Revision 1.3  1995/09/29 17:32:31  steve
 *  Define NS resolution of ticker.
 *
 * Revision 1.2  1995/09/25 00:29:22  steve
 *  Optimize ISR selection and dispatch, and try
 *  to speed up handling of time.
 *
 * Revision 1.1  1995/09/02 18:27:20  steve
 *  Initial import of UCR core to cvs.
 */

# include  <mc68322/TimerModule.h>
# include  <ucr/ucr.h>
# include  <assert.h>

/*
 * Setup register pointers to access the device, clear any pending
 * interrupt and disable support for them. The start() method will
 * enable interrupts.
 *
 * NOTE: The ns_per_tick of 50*interval assumes that CLK1 is 20MHz. I
 * really should fix this.
 */

TimerModule::TimerModule(unsigned char level, unsigned long interval,
			 unsigned long base)
: Ticker(interval*50), regs_( (REGS_*) (base + 0x600) ),
  intr_( (INTR_*) (base + 0x730) )
{
      intr_->enable_flag = 0;
      intr_->int_flag = 1;
      intr_->level = level;
      regs_->interval = interval;
      set_isr(level, isr, this);
}

TimerModule::~TimerModule()
{
      intr_->enable_flag = 0;
      intr_->int_flag = 1;
}

void TimerModule::start()
{
      intr_->int_flag = 1;
      intr_->enable_flag = 1;
}

/*
 * Handle the Timer interrupt by clearing it and calling the
 * handle_tick() method of the base class, Ticker.
 */
void TimerModule::isr(TimerModule*obj, unsigned)
{
      if (obj->intr_->int_flag) {
	    obj->intr_->int_flag = 1;
	    obj->handle_tick();
      }
}
