/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: fputs.cc,v 1.3 1996/10/28 03:46:03 steve Exp $"
/*
 * $Log: fputs.cc,v $
 * Revision 1.3  1996/10/28 03:46:03  steve
 *  License notice.
 *
 * Revision 1.2  1996/09/16 17:03:08  steve
 *  Add the read/write state machine.
 *
 * Revision 1.1  1996/09/14 00:10:20  steve
 *  Add a stdio library.
 *
 */

# include  "stdiox.h"
# include  <string.h>
# include  <assert.h>

extern "C" int fputs(const char*str, FILE*stream)
{
      assert( stream->flags & _FILE::F_WRITE );
      assert( stream->state != _FILE::S_READ );

      if (stream->state == _FILE::S_ERROR) return EOF;
      stream->state = _FILE::S_WRITE;

      size_t len = strlen(str);

      switch (stream->buf_mode) {

	      // Move bytes to the buffer until the buffer is full or
	      // I run out of buffers. Each time around, if the buffer
	      // is full write it out. Do this until the entire string
	      // is written.
	  case _IONBF:
	    stream->dev->write(str, len);
	    break;

	  case _IOFBF:
	    stream->write_to_buf(str, len);
	    break;

	  case _IOLBF:
	    while (*str) fputc(*str++, stream);
	    break;

      }
      return len;
}
