/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: sprintf.cc,v 1.4 1996/10/28 03:46:05 steve Exp $"
/*
 * $Log: sprintf.cc,v $
 * Revision 1.4  1996/10/28 03:46:05  steve
 *  License notice.
 *
 * Revision 1.3  1996/10/24 00:04:20  steve
 *  NUL terminate the result.
 *
 * Revision 1.2  1996/09/16 19:34:31  steve
 *  Support sprintf.
 *
 * Revision 1.1  1996/09/16 17:02:35  steve
 *  Add fprintf and sprintf.
 *
 */

# include  "stdiox.h"

/*
 * I use the _vformat function to write into the string. The trouble
 * is that function is written to deliver characters to a writeable
 * stream. I deal with that by bumming up the following trivial
 * replacement for fputc that writes into memory instead of a
 * stream. I use a FILE structure to carry the context, but I do not
 * use it as a file.
 */

static int putbuff(int c, FILE*str)
{
      *(str->buf_ptr++) = c;
      return c;
}

extern "C" int sprintf(char*str, const char*format, ...)
{
      va_list arg;
      va_start(arg, format);

	// Make up a fake FILE object to pass to _vformat. I only need
	// to fill in the stuff that putbuff uses.
      FILE tmp;
      tmp.buf_ptr = str;

      int result = _vformat(putbuff, &tmp, format, arg);
      putbuff(0, &tmp);

      (void)va_end(arg);

      return result;
}
