#ifndef ___thread_H
#define ___thread_H

/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: __thread.h,v 1.12 1997/08/25 21:24:21 steve Exp $"
/*
 * $Log: __thread.h,v $
 * Revision 1.12  1997/08/25 21:24:21  steve
 *  Add some stack thread checks.
 *
 * Revision 1.11  1996/11/08 21:40:35  steve
 *  Divide up thread_create for easier porting, Add the
 *  sparc_linux and i386_linux targets, and adjust makefiles
 *  to allow for them. Document the new concept of virtual boards.
 *
 *  Change the linker output file extension to .exe, because
 *  some targets use ELF and not COFF.
 *
 *  Get rid of some trivial warnings.
 *
 * Revision 1.10  1996/10/28 03:46:15  steve
 *  License notice.
 *
 * Revision 1.9  1996/09/19 04:44:09  steve
 *  Add the i960_ise target, change the target
 *  configuration headers to allow sharing by
 *  targets, fix up the make clean rule.
 *
 * Revision 1.8  1996/09/17 21:51:38  steve
 *  Add support for errno.h
 *
 * Revision 1.7  1996/08/25 00:29:11  steve
 *  Optimize thread switching.
 *
 * Revision 1.6  1996/08/14 00:24:48  steve
 *  Add fault information to the THREAD object so that
 *  it can be retrieved at any time.
 *
 * Revision 1.5  1996/08/03 03:30:40  steve
 *  Create the ucr/asm.h header as a target specific
 *  header with line functions, etc; Replace the
 *  __isr_run_flag with a bit of inline assembly code.
 *
 *  Move much thread code out of the __Secheduler class
 *  and into more public thread_ functions, to eliminate
 *  some wasted function call depth. Even the main_thread
 *  is move into its own file.
 *
 *  Keep a list of available threads for debug use, and
 *  add a function to get thread registers.
 *
 * Revision 1.4  1996/07/19 04:29:23  steve
 *  change ISR handling to do thread switching when the return
 *  from interrupt happens.
 *
 * Revision 1.3  1996/05/27 22:50:01  steve
 *  implement thread_create and thread_switch for the i960,
 *  rename some __foo symbols to uCR_foo for neatness.
 *
 * Revision 1.2  1995/10/06 00:07:50  steve
 *  support stack overflow checks.
 *
 * Revision 1.1  1995/09/02 18:27:28  steve
 *  Initial import of UCR core to cvs.
 *
 *
 */

/*
 * This is a private header file that contains internal data types and
 * declarations for the implementation of threads.
 */

#if defined(m68k_coff)
# include "thread_cookie-m68k.h"
#endif

#if defined(i960_cyclone) || defined(i960_ise)
# include "thread_cookie-i960.h"
#endif

#if defined(i386_linux)
# include  "thread_cookie-i386.h"
#endif

#if defined(sparc_linux)
# include  "thread_cookie-sparc.h"
#endif

#ifndef __thread_cookie_H
# error "I do not know this CPU type."
#endif

/*
 * This structure is placed at the top of the stack for a thread. When
 * the thread is suspended, this structure contains the registers,
 * including the stack pointer. The stack pointer points to the bottom
 * of the stack which contains the pc for the next instruction.
 *
 * the fault_ pointer points to the FAULT_RECORD for the threads
 * current fault. This is set to 0 until it enters the fault
 * handler. If the pointer is anything other then 0, then the thread
 * is processing a fault of some sort.
 *
 * The THREAD objects are all kept is a circular list that can be
 * traversed by the debuggers. uCR in no way uses the global list.
 *
 * NOTE: The uCR_switch.s source file contains assembler references to
 * regs and sreg within this structure. It uses absolute offsets from
 * the beginning, so regs and sreg must not move.
 *
 * NOTE: Most targets assume that ThreadCookie is the first member of
 * the THREAD struct. Some assembly code accesses it.
 */

struct ThreadQueue;
struct FAULT_RECORD;

struct THREAD {
      ThreadCookie cookie;

        // Pointers to the stack. For the main stack, *stack is not
	// filled in. They are left nil and the main thread cannot be
	// deleted.
      void*bstack;
      void*estack;

	// The C library expects an errno accessible to the thread. I
	// use this to make errno UNIQUE to the thread.
      int errno_;

      struct FAULT_RECORD*fault_;

      struct THREAD *next_, *prev_;
      struct ThreadQueue *owner_;

	// The created threads all go into a list. The debugger
	// support uses this list for finding threads of note.
      struct THREAD *list_;
};

/*
 * There are a few cases where assembly code may desire access to the
 * current thread pointer. So, I make it available globally. I use C
 * linkage so that assembly code can get at it if necessary.
 */
extern struct THREAD*uCR_current_thread;

/*
 * If the thread_cookie header file defines the name of a register
 * that contains the current stack pointer, use that to generate a
 * stack check function. This function returns true if the stack
 * pointer is outside the range of the stack.
 */
#if defined(CPU_STACK_REGISTER)
static inline bool uCR_thread_stack_check()
{
      register void*sp asm(CPU_STACK_REGISTER);
      return (uCR_current_thread->bstack >= sp)
	    || (uCR_current_thread->estack <= sp);
}
#else
#define uCR_thread_stack_check() (false)
#endif

/*
 * This is a more primitive form of switch, that is used by some of
 * the internal stuff when run Q manipulation is already handled.
 *
 * Note that this method must be protected by spl calls. It is a
 * private method, and all the public methods that call it take care
 * of safety.
 */

extern "C" void uCR_switch(ThreadCookie*to, ThreadCookie*self);

inline void thr_switch_(THREAD*tid)
{
      THREAD*self = uCR_current_thread;
      uCR_current_thread = tid;
      uCR_switch(&tid->cookie, &self->cookie);
}

#endif
