/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
 .ident "$Id: locore-i960_cyclone.s,v 1.13 1997/08/21 21:21:25 steve Exp $"
/*
 * $Log: locore-i960_cyclone.s,v $
 * Revision 1.13  1997/08/21 21:21:25  steve
 *  Support replaceable main stack.
 *
 * Revision 1.12  1996/10/28 03:46:26  steve
 *  License notice.
 *
 * Revision 1.11  1996/08/27 03:06:41  steve
 *  Make fault calls into system calls.
 *
 * Revision 1.10  1996/08/22 00:35:42  steve
 *  main stack has beginning and end.
 *
 * Revision 1.9  1996/08/18 05:28:24  steve
 *  Do trace faults with system-supervisor call
 *
 */

.file "locore.s"

 # These are some initial values for various i960 registers.

	.set	MMR_IMSK, 0xff008504
	.set	IMSK_DEFAULT,  0x30ff

	.set	TRACE_CALL_NUM, 200

 # The cyclone board has mon960 running on it, so some things I do
 # as calls to the monitor. These are system call numbers for the
 # system calls I use.

	.set	sys_get_prcbptr, 245

	.globl	start
	.text
start:

 # Disable interrupts. The ucr_start routine will re-enable them with
 # a call to thread_spl_low().

	lda     0x001f0001, g0
	modpc	g0, g0, g0

 # The mon960 monitor gives me only a small stack, just big enough to
 # start up. Here, I set up the stack frame that I really want to use.

	lda	_bstack, g1
	cmpobe	g1, fp, 0f
	mov	g1, sp
	call	0f
0:	flushreg
	lda	0, pfp
	
 # Write into the IMSK register to turn on the interrupts that the
 # application can be expected to receive. However, leave the priority
 # high so they don't actually happen.

	lda	MMR_IMSK, g0
	ldconst IMSK_DEFAULT, g1
	atmod	g0, g1, g1

 # Get the PRCB pointer from mon960, get the interrupt vector table
 # out of that, and load all the interrupt vectors from there. The cyclone
 # bootstrap sets up specific values for these numbers, so I just accept
 # these numbers and program them in.

	ldconst	sys_get_prcbptr, r3
	calls	r3


 # These are the interrupt table entries. Point them to uCR_isr_common.

	ld	16(g0), g1	# Interrupt table address..
	lda	_uCR_isr_common, g2
	st	g2, 0x62*4+4(g1)
	st	g2, 0x72*4+4(g1)
	st	g2, 0x82*4+4(g1)
	st	g2, 0x92*4+4(g1)
	st	g2, 0xa2*4+4(g1)
	st	g2, 0xb2*4+4(g1)
	st	g2, 0xc2*4+4(g1)
	st	g2, 0xd2*4+4(g1)
	st	g2, 0xe2*4+4(g1)
	st	g2, 0xf2*4+4(g1)

 # These are the fault table entries. Point them to system call
 # TRACE_CALL_NUM, which will call _uCR_fault_common.

	ld	00(g0), g1	# Fault table address...
	ldconst	TRACE_CALL_NUM*4+2, g4
	ldconst	0x27f, g5

	stl	g4, 0*8(g1)	# OVERRIDE/PARALLEL FAULT
	stl	g4, 1*8(g1)	# TRACE FAULT
	stl	g4, 2*8(g1)	# OPERATION FAULT
	stl	g4, 3*8(g1)	# ARITHMETIC FAULT
	stl	g4, 5*8(g1)	# CONSTRAINT FAULT
	stl	g4, 7*8(g1)	# PROTECTION FAULT
	stl	g4,10*8(g1)	# TYPE FAULT

 # Write the address of uCR_fault_common into the system call
 # table entry for the appropriate system call number, and mark
 # it as a system-supervisor call.

	lda	_uCR_fault_common, g2
	ld	20(g0), g1
	or	g2, 0x2, g3
	st	g3, TRACE_CALL_NUM*4+0x30(g1)

 # Turn the trace bit OFF when it enters any system call. I can turn it
 # on later (within the call) if I want to, but I probably won't. This
 # prevents fault handlers being traced.

	ld	12(g0), g1
	notand	g1, 1, g1
	st	g1, 12(g0)

 # All set. Start the application here.

	call	_ucr_start

 # The call to ucr_start returns int. When I return, call the exit
 # entry of the mon960 monitor. Leave the return code in g0, which
 # becomes the first parameter of the _exit call.

	ldconst	257, r3
	calls	r3
