
/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: stack_trace-m68k_coff.cc,v 1.2 1996/10/28 03:46:35 steve Exp $"
/*
 * $Log: stack_trace-m68k_coff.cc,v $
 * Revision 1.2  1996/10/28 03:46:35  steve
 *  License notice.
 *
 * Revision 1.1  1996/05/29 23:08:06  steve
 *  Seperate stack_trace to arcitecture files.
 *
 */

/*
 * This function fills a buffer with a stack trace, starting from the
 * caller.
 *
 * The Stack looks like this:
 *
 *          | %pc   |
 *	    | %fp   | <--+
 *	    | local |    |
 *		         |
 *	    | args  |    |
 *	    | %pc   |    |
 * %fp ---> | %fp   |  --+
 *	    | local |
 *
 * Each frame pointer points to the previous frame pointer that is
 * saved on the stack. This structure is common to many (most?) CPU
 * architectures and compilers, so this should be fairly easy to port.
 *
 * The stack_trace routine follows the frame pointers and returns all
 * the program counters. These are return addresses, so the %pc is
 * frame 1 is really the address is frame 2 that called into frame
 * 1. This affects stack traces of interrupted threads, to wit: the
 * frame of the interrupted function doesn't necessarily show up in a
 * trace.
 *
 *          | %pc   |
 *	    | %fp   | <--+  Interrupted function
 *	    | local |    |
 *		         |
 *          | %pc[2]|
 *          | %sr   | Interrupt handler base frame
 *          | regs  |
 *
 *	    | args  |    |
 *	    | %pc[1]|    |  C++ ISR frame
 * %fp ---> | %fp   |  --+
 *	    | local |
 *
 * A stack_trace called in the C++ ISR will show the call from the
 * handler base frame (%pc[1]) but will not show the PC into the
 * interrupted function because the interrupt handler base frame does
 * not look like a stack frame, and does not link.
 *
 * To do so the handler base would have to pull a copy of the %pc[2]
 * to the top of the stack, then do a link %fp,#0 to insert itself as
 * a stack frame. One of these days I might do that.
 *
 * NOTE: All this will not work if you use -fomit-frame-pointer!
 * Functions compiled with that flag will vanish from a stack trace,
 * if you are lucky. If the compiler allocates something to the %fp,
 * the stack_trace search with trace itself into oblivion.
 */


# include  <assert.h>

unsigned stack_trace(unsigned long*buffer, unsigned bsize)
{
        // Start with my own frame pointer. (The assert proves that
	// the frame pointer really points to my frame.)
      unsigned long*frame;
      asm("movel %%fp,%0" : "=d" (frame) );
      assert( frame[2] == (unsigned long)buffer );

        // Go to the caller's frame, and start looking for PCs.
      unsigned count = 0;

      while (*frame && (count < bsize)) {
	    buffer[count] = frame[1];
	    count += 1;
	    frame = (unsigned long*)frame[0];
      }

      return count;
}
