#ifndef __thread_cookie_H
#define __thread_cookie_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: thread_cookie-i386.h,v 1.2 1996/11/09 04:34:25 steve Exp $"
/*
 * $Log: thread_cookie-i386.h,v $
 * Revision 1.2  1996/11/09 04:34:25  steve
 *  Add i386_linux thread infrastructure.
 *
 * Revision 1.1  1996/11/08 21:40:43  steve
 *  Divide up thread_create for easier porting, Add the
 *  sparc_linux and i386_linux targets, and adjust makefiles
 *  to allow for them. Document the new concept of virtual boards.
 *
 *  Change the linker output file extension to .exe, because
 *  some targets use ELF and not COFF.
 *
 *  Get rid of some trivial warnings.
 *
 */

/*
 * The i386 target thread registers are the EX registers, and the
 * stack and index registers.
 *
 * NOTE: Since interrupts are not yet supported, the CC flags are not
 * important (the compiler knows they will be trashed) and I do not
 * save the PSW. When I do a real i386 implementation, I will need to
 * add status register switching.
 */
# define REGISTER_COUNT 8

# define I386_REG_EAX 0
# define I386_REG_EDX 1
# define I386_REG_ECX 2
# define I386_REG_EBX 3
# define I386_REG_EBP 4
# define I386_REG_ESI 5
# define I386_REG_EDI 6
# define I386_REG_ESP 7

struct ThreadCookie {
      unsigned long regs [REGISTER_COUNT];
};

# define COOKIE_INITIALIZER { { 0 } }

static inline void thread_cookie_init(ThreadCookie*cookie)
{
      for (unsigned idx = 0 ;  idx < REGISTER_COUNT ;  idx += 1)
	    cookie->regs[idx] = 0;
}



#endif
