
#
# Copyright (c) 1996 Picture Elements Inc.
#	Stephen Williams
#
#    This source code is free software; you can redistribute it
#    and/or modify it in source code form under the terms of the GNU
#    Library General Public License as published by the Free Software
#    Foundation; either version 2 of the License, or (at your option)
#    any later version. In order to redistribute the software in
#    binary form, you will need a Picture Elements Binary Software
#    License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Library General Public License for more details.
#
#    You should have received a copy of the GNU Library General Public
#    License along with this program; if not, write to the Free
#    Software Foundation, Inc.,
#    59 Temple Place - Suite 330
#    Boston, MA 02111-1307, USA
#
#    You should also have recieved a copy of the Picture Elements
#    Binary Software License offer along with the source. This offer
#    allows you to obtain the right to redistribute the software in
#    binary (compiled) form. If you have not received it, contact
#    Picture Elements, Inc.,
#    777 Panoramic Way
#    Berkeley, CA 94704.
#
.ident "$Id: uCR_isr-i960.s,v 1.3 1996/10/28 03:46:45 steve Exp $"
#
# $Log: uCR_isr-i960.s,v $
# Revision 1.3  1996/10/28 03:46:45  steve
#  License notice.
#
# Revision 1.2  1996/10/04 01:31:25  steve
#  Manage the IMSK register.
#
# Revision 1.1  1996/09/19 04:44:20  steve
#  Add the i960_ise target, change the target
#  configuration headers to allow sharing by
#  targets, fix up the make clean rule.
#
# Revision 1.9  1996/08/24 20:57:38  steve
#  The priority cookie is not necessarily short, so make
#  a typedef for it.
#
#  There is no need (indeed no way) to save and restore the
#  fp of a thread, so put the PFP, which is really the fp,
#  into the fp register slot.
#
#  Slightly better check of the stack frame at thread switch
#  time. Also have faults check for some bad things.
#
# Revision 1.8  1996/08/22 00:36:12  steve
#  the stack pointer is not handled seperately.
#
# Revision 1.7  1996/08/18 22:36:25  steve
#  Switch hte TC register with the thread.
#
# Revision 1.6  1996/08/13 03:30:40  steve
#  Fix handling of frame when switching threads.
#
# Revision 1.5  1996/08/02 23:00:17  steve
#  Clear IPND manually, just in case.
#
# Revision 1.4  1996/07/19 23:47:38  steve
#  Calling thread_pause in an isr is not OK. Also,
#  make thread switches in ISRs OK. The assembly
#  code hides the fact that that switch actually
#  takes affect when the ISR returns.
#
# Revision 1.3  1996/07/19 04:29:24  steve
#  change ISR handling to do thread switching when the return
#  from interrupt happens.
#
# Revision 1.2  1996/05/29 22:07:24  steve
#  Properly switch pc register with threads.
#
# Revision 1.1  1996/05/29 04:36:07  steve
#  Support interrupts in the i960_cyclone world, and in the
#  process reorganize interrupt code for portability.
#
# Revision 1.1  1996/05/24 01:46:42  steve
#  Initial port to i960_cyclone
#  Seperate some files in the library to seperate files
#  for making smaller binaries, use uCR_ prefix for private
#  symbols, move assembly files around for compile-time
#  portability, change __start to ucr_start and give it
#  new behavior, and simplify the ucr_assert implementation.
#
#
	.file	"ucr_isr.s"

	.set	COOKIE_PFP, 60
	.set	COOKIE_PC,  64
	.set	COOKIE_AC,  68
	.set	COOKIE_TC,  72

	.set	IPND, 0xff008500
	.set	IMSK, 0xff008504

# NOTE:	The interrupt vector table points to this. It is very
# incorrect to call this explicitly, as return behavior is crafted to
# return from interrupts.

# NOTE:	This code assumes that the i960 is configured to clear the IMSK
# and leave the old IMSG value in r3. I use that feature to protect the
# code from interrupt while I raise the processor priority. Even if the
# i960 is not configured to clear the IMSK, the st of r3 into IMSK will
# not hurt anything.


	.global _uCR_isr_common
_uCR_isr_common:

 # Disable further interrupts. The return from interrupts will restore
 # the interrupt level by reloading the process controlls register.

	shlo	16,0x1f, r4	# ldconst 0x001f0000, r4
	modpc	r4, r4, r4
	st	r3, IMSK
	
 # This is called at interrupt time, so the caller does not know to
 # save g* registers. That means, I have to do it because there is no
 # reasonable way to prevent the called code mussing with them.

	ld	_uCR_current_thread, r4
	stq	g0,  0(r4)
	stq	g4, 16(r4)
	stq	g8, 32(r4)
	stt	g12,48(r4)
	st	pfp, COOKIE_PFP(r4)

 # Get the saved PC and AC registers from the interrupt stack frame
 # and the current TC from the TC register, and save them in the
 # thread cookie. I will restore them later.

	ldl	-16(fp), g0
	modtc	0, 0, g2
	stt	g0, COOKIE_PC(r4)

 # The cyclone board uses vector numbers pppp0010 where pppp goes from
 # 0110 to 1111. Convert these to the numbers 0-10 by a clever little
 # bit of math.

	ldob	-8(fp), g0
	shro	4, g0, g0
	subo	6, g0, g0

	mov	0, g14
	call	_uCR_isr_base

 # Clear the IPND register explicitly. If there are any interrupts
 # still pending after all, then that bit will remain set in the
 # IPND register in spite of what I write into it.

	mov	0, r6
	st	r6, IPND

 # Look at the current thread pointer. If the value is different from
 # when I entered the ISR, then the thread must be switched. This
 # involves the icky task of changing the previous frame pointer. Pay
 # special attention that the return-type bits of the PFP reflect that
 # I am returning form an interrupt.

	ld	_uCR_current_thread, r5

	ldt	COOKIE_PC(r5), g0
	stl	g0, -16(fp)

	cmpobe	r4, r5, 0f

	ldconst	0xfe, g0
	modtc	g0, g2, g2
	flushreg
	ld	COOKIE_PFP(r5), pfp
	or	pfp, 0x07, pfp
	flushreg
0:

 # This stuff must be done whether the thread is switched or not. Just
 # restoring the global register. The PC and AC are now on the stack,
 # and the local registers are in the stack frame referenced by pfp.

	ldt	48(r5), g12
	ldq	32(r5), g8
	ldq	16(r5), g4
	ldq	 0(r5), g0

	ret
