
#
# Copyright (c) 1996 Picture Elements Inc.
#	Stephen Williams
#
#    This source code is free software; you can redistribute it
#    and/or modify it in source code form under the terms of the GNU
#    Library General Public License as published by the Free Software
#    Foundation; either version 2 of the License, or (at your option)
#    any later version. In order to redistribute the software in
#    binary form, you will need a Picture Elements Binary Software
#    License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Library General Public License for more details.
#
#    You should have received a copy of the GNU Library General Public
#    License along with this program; if not, write to the Free
#    Software Foundation, Inc.,
#    59 Temple Place - Suite 330
#    Boston, MA 02111-1307, USA
#
#    You should also have recieved a copy of the Picture Elements
#    Binary Software License offer along with the source. This offer
#    allows you to obtain the right to redistribute the software in
#    binary (compiled) form. If you have not received it, contact
#    Picture Elements, Inc.,
#    777 Panoramic Way
#    Berkeley, CA 94704.
#
.ident "$Id: uCR_switch-i960.s,v 1.2 1996/10/28 03:46:46 steve Exp $"
#
# $Log: uCR_switch-i960.s,v $
# Revision 1.2  1996/10/28 03:46:46  steve
#  License notice.
#
# Revision 1.1  1996/09/19 04:44:21  steve
#  Add the i960_ise target, change the target
#  configuration headers to allow sharing by
#  targets, fix up the make clean rule.
#
# Revision 1.9  1996/08/24 20:57:39  steve
#  The priority cookie is not necessarily short, so make
#  a typedef for it.
#
#  There is no need (indeed no way) to save and restore the
#  fp of a thread, so put the PFP, which is really the fp,
#  into the fp register slot.
#
#  Slightly better check of the stack frame at thread switch
#  time. Also have faults check for some bad things.
#
# Revision 1.8  1996/08/22 00:36:12  steve
#  the stack pointer is not handled seperately.
#
# Revision 1.7  1996/08/18 22:36:26  steve
#  Switch hte TC register with the thread.
#
# Revision 1.6  1996/08/13 03:27:33  steve
#  Make sure PFP.rt is always local return.
#
# Revision 1.5  1996/07/19 23:40:28  steve
#  Slight optimization
#
# Revision 1.4  1996/07/19 04:29:25  steve
#  change ISR handling to do thread switching when the return
#  from interrupt happens.
#
# Revision 1.3  1996/05/29 22:52:53  steve
#  Simplify and optimize i960 thread switching.
#
# Revision 1.2  1996/05/29 22:07:25  steve
#  Properly switch pc register with threads.
#
# Revision 1.1  1996/05/27 22:50:03  steve
#  implement thread_create and thread_switch for the i960,
#  rename some __foo symbols to uCR_foo for neatness.
#
# Revision 1.1  1996/05/24 01:46:44  steve
#  Initial port to i960_cyclone
#  Seperate some files in the library to seperate files
#  for making smaller binaries, use uCR_ prefix for private
#  symbols, move assembly files around for compile-time
#  portability, change __start to ucr_start and give it
#  new behavior, and simplify the ucr_assert implementation.
#
#


	.set	COOKIE_PFP, 60
	.set	COOKIE_PC,  64
	.set	COOKIE_AC,  68
	.set	COOKIE_TC,  72

# Save the registers in the self thread structure, and get new
# registers from the new thread structure. The switch includes
# the stack pointer, and so also includes the stack itself. This
# function assumes that the PC that I'm switching out blocks interrupts,
# but it is possible that the PC i'm switching in does not.

# The i960 calling sequence has
#	g0 == (ThreadCookie*)to
#	g1 == (ThreadCookie*)self
#
	.global _uCR_switch
_uCR_switch:

 # This extra call makes a little space on the stack for writing things
 # before the frame pointer. The loading of the PC register uses this,
 # so see below for how that works. The actual contents of the local
 # registers of this fake frame do not matter and will be trashed.
	call	0f
	ret
0:

 # WATCH OUT! If I am in an interrupt handler, then isr_common will
 # take care of switching the thread when it returns from the
 # interrupt stack. The following test will cause the switch to be
 # aborted if the PC.s bit is set.

	modpc	0, 0, r4
	bbc	13, r4, 0f
	ret
	
 # Save the current registers in self->regs, including the current
 # process control register. There is no need to save the local registers
 # because the caller's registers are already safe, and mine are
 # of no value.
0:
	modac	0, 0, r5
	modtc	0, 0, r6
	stt	r4, COOKIE_PC(g1)

	stq	 g0,   (g1)
	stq	 g4, 16(g1)
	stq	 g8, 32(g1)
	stt	g12, 48(g1)
	st	pfp, COOKIE_PFP(g1)

 # Switch to the stack of the new thread.

 # Note that I choose to pretend to be an interrupt return. This allows
 # me to write the pc and ac registers into the stack instead of into
 # the register itself.

	flushreg
	ld	COOKIE_PFP(g0), pfp
	or	pfp, 0x07, pfp
	flushreg

 # Load the registers from the new thread, collect the processor
 # state into local registers.

	ldt	COOKIE_PC(g0), r4

	ldt	48(g0), g12
	ldq	32(g0), g8
	ldq	16(g0), g4
	ldq	 0(g0), g0

 # Finally, load the processor state into the right places on the stack,
 # load the trace controls, and return to the new thread. The return will
 # atomically load the fp from the pfp (thus switching the stack) and
 # load the PC register to match the new thread.

	ldconst	0xfe, r3
	modtc	r3, r6, r6
	stl	r4, -16(fp)

	ret		# Return to the new thread

# If the other thread was departed by a uCR_switch, then right here the
# other thread will resume as if it just executed this return and will
# go to the return up above. If the other thread was departed by an
# interrupt handler switching the thread, then the return above returns
# the thread to the point where it was interrupted.
