/*
 * Copyright (c) 1997 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *  ---
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 */
#ident "$Id: gensyms.c,v 1.1 1997/12/18 22:53:04 steve Exp $"

# include  <stdio.h>
# include  <string.h>
# include  <stdlib.h>

/*
 * This program reads from stdin strings that are symbol names. The
 * output is assembler source that compiles into a symbol image, and
 * can be linked with a program that uses udlopen.
 */

struct name_cell {
      struct name_cell*next;
      char* name;
};

int main(int argc, char*argv[])
{
      char line[1024];
      struct name_cell*list = 0, *cur;
      unsigned nlist = 0;

      while (fgets(line, sizeof line, stdin)) {
	    char*np;
	    if (np = strchr(line, '\n')) *np = 0;
	    cur = calloc(1, sizeof(struct name_cell));
	    cur->name = strdup(line);

	    if ((list == 0) || (strcmp(cur->name,list->name) < 0)) {
		  cur->next = list;
		  list = cur;

	    } else {
		  struct name_cell*idx;

		  for (idx = list ;  idx->next ;  idx = idx->next)
			if (strcmp(cur->name,idx->next->name) <= 0) break;

		  cur->next = idx->next;
		  idx->next = cur;
	    }
	    nlist += 1;
      }

      printf(".text\n");
      printf("$ymbol_table: .global $ymbol_table\n");
      printf("\t.word 0x316d7973\n");
      printf("\t.word symbol_table_end - $ymbol_table\n");
      printf("\t.word %u\n", nlist);

      for (cur = list ;  cur ;  cur = cur->next) {
	    printf("\t.word _%s\n", cur->name);
      }

      for (cur = list ;  cur ;  cur = cur->next) {
	    printf("\t.asciz \"_%s\"\n", cur->name);
      }

      printf("symbol_table_end:\n");

      return 0;
}

/*
 * $Log: gensyms.c,v $
 * Revision 1.1  1997/12/18 22:53:04  steve
 *  Translate to C, for no good reason.
 *
 * Revision 1.1  1997/09/03 23:45:03  steve
 *  Add support for dynamic loading, including the
 *  commands to make symbol tables, read and link
 *  modules, and download fondled binaries.
 *
 */

