/*
 * Copyright (c) 1997 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *  ---
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 */
#ident "$Id: getsyms.cc,v 1.2 1997/09/04 01:05:45 steve Exp $"

# include  <stdio.h>
# include  <fcntl.h>
# include  <assert.h>
# include  <unistd.h>
# include  <stdlib.h>
# include  <unistd.h>
# include  <ucrif.h>

const char*path = "/dev/ise0";
unsigned long channel = 0;
unsigned long symboltable = 0;

struct Head {
      unsigned long cmd;
      unsigned long parm1, parm2;
      unsigned long len;
};

/*
 * cmd 1  -- GET SYMBOLS
 *  parm1 = 0;
 *  parm2 = 0;
 *  len = 0;
 *    returns a symbol table
 *
 * cmd 2  -- GET ALLOCATION
 *  parm1 is required size;
 *  parm2 = 0;
 *  len = 0;
 *
 * cmd 3 -- SEND IMAGE
 *  parm1 is address of image
 *  parm2 is address of symbol table in image, or 0 for no table.;
 *  len = size of image
 */


static int get_symbols()
{
      int fd = open(path, O_RDWR, 0);
      assert( fd > -1 );

      int rc = ioctl(fd, UCR_CHANNEL, channel);
      assert( rc > -1 );

	// Send the command requesting symbols
      Head head;
      head.cmd = 1;
      head.parm1 = 0;
      head.parm2 = 0;
      head.len = 0;
      rc = write(fd, &head, sizeof head);
      assert( rc == sizeof head );
      rc = ioctl(fd, UCR_FLUSH, 0);
      assert( rc > -1 );

	// Receive the symbol table from the target.
      struct Info {
	    unsigned long magic;
	    unsigned long size;
	    unsigned long count;
      } info;

      rc = read(fd, &info, sizeof info);

      unsigned long*vals = new unsigned long[info.count];
      rc = read(fd, vals, info.count * sizeof(unsigned long));

      size_t strings_sz = info.size - info.count*sizeof(long) - sizeof info;
      char*strings = new char[strings_sz];
      rc = read(fd, strings, strings_sz);

      close(fd);

      unsigned long*vp = vals;
      char*cp = strings;
      for (unsigned idx = 0 ;  idx < info.count ;  idx += 1) {
	    printf("%s = 0x%x;\n", cp, *vp);
	    vp += 1;
	    cp += strlen(cp) + 1;
      }

      return 0;
}

static int get_allocate(const char*val)
{
      if (val == 0) {
	    fprintf(stderr, "missing numeric value.\n");
	    return 1;
      }

      int fd = open(path, O_RDWR, 0);
      assert( fd > -1 );

      int rc = ioctl(fd, UCR_CHANNEL, channel);
      assert( rc > -1 );

      Head head;
      head.cmd = 2;
      head.parm1 = strtoul(val, 0, 0);
      head.parm2 = 0;
      head.len = 0;
      rc = write(fd, &head, sizeof head);
      rc = ioctl(fd, UCR_FLUSH, 0);


      struct mod_info {
	    void*base;
	    unsigned long size;
      } info;

      rc = read(fd, &info, sizeof info);

      printf("%p %lu\n", info.base, info.size);
      return 0;
}

static int send_image(int argc, char*argv[])
{
      if (argc != 2) {
	    fprintf(stderr, "wrong number of parms.\n");
	    return -1;
      }

      int image_fd = open(argv[1], O_RDONLY, 0);
      if (image_fd == -1) {
	    perror(argv[1]);
	    return 1;
      }

      int fd = open(path, O_RDWR, 0);
      assert( fd > -1 );

      int rc = ioctl(fd, UCR_CHANNEL, channel);
      assert( rc > -1 );

      size_t size_of_image = lseek(image_fd, 0, SEEK_END);
      lseek(image_fd, 0, SEEK_SET);

      void*image = malloc(size_of_image);
      if (image == 0) {
	    close(image_fd);
	    fprintf(stderr, "not enough memory.\n");
	    return 1;
      }

      rc = read(image_fd, image, size_of_image);
      assert( rc == size_of_image );

      Head head;
      head.cmd = 3;
      head.parm1 = strtoul(argv[0], 0, 0);
      head.parm2 = symboltable;
      head.len = size_of_image;

      rc = write(fd, &head, sizeof head);
      rc = write(fd, image, size_of_image);
      rc = ioctl(fd, UCR_FLUSH, 0);

      free(image);
      close(fd);
}

int main(int argc, char*argv[])
{
      int opt;
      while ((opt = getopt(argc, argv, "S:c:p:")) != EOF) switch(opt) {

	  case 'S':
	    symboltable = strtoul(optarg, 0, 0);
	    break;
	  case 'c':
	    channel = strtoul(optarg, 0, 0);
	    break;

	  case 'p':
	    path = optarg;
	    break;

	  default:
	    fprintf(stderr, "Usage: %s [-c <n>] [-p <path>] "
		    "symbols|allocate|image\n");
	    return -1;
      }

      if ((argc-optind) < 1) {
	    fprintf(stderr, "missing subcommand.\n");
	    return -1;
      }

      if (strcmp("symbols",  argv[optind]) == 0)
	    return get_symbols();

      if (strcmp("allocate", argv[optind]) == 0)
	    return get_allocate(argv[optind+1]);
      if (strcmp("image",    argv[optind]) == 0)
	    return send_image(argc-optind-1, argv+optind+1);
      return -1;
}

/*
 * $Log: getsyms.cc,v $
 * Revision 1.2  1997/09/04 01:05:45  steve
 *  Add the udlsym function.
 *
 * Revision 1.1  1997/09/03 23:45:04  steve
 *  Add support for dynamic loading, including the
 *  commands to make symbol tables, read and link
 *  modules, and download fondled binaries.
 *
 */

