#! /bin/sh
# Copyright (c) 1997 Picture Elements, Inc.
# Stephen Williams (steve@picturel.com)
#
# $Id: ucrmod.sh,v 1.3 1997/09/04 01:15:43 steve Exp $
#
#    This source code is free software; you can redistribute it
#    and/or modify it in source code form under the terms of the GNU
#    General Public License as published by the Free Software
#    Foundation; either version 2 of the License, or (at your option)
#    any later version. In order to redistribute the software in
#    binary form, you will need a Picture Elements Binary Software
#    License.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
#
#    You should also have recieved a copy of the Picture Elements
#    Binary Software License offer along with the source. This offer
#    allows you to obtain the right to redistribute the software in
#    binary (compiled) form. If you have not received it, contact
#    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.

# This program is the driver for performing the actual run-time link
# of the object files (modules) onto the target board. It communicates
# with the target board (using getsyms) to get the symbol table
# information, the links the object into

tmp=,mod$$

libdirs=-L$UCR/$TARGET/lib
libs=
path=/dev/ise0
channel=0
exe=$tmp.exe
myself=`basename $0`

set -- `getopt c:L:l:o:p: $*`
for i
do
    case "$i"
    in
	-c)
	    channel=$2
	    shift ; shift ;;
	-L)
	    libdirs="$libdirs -L$2"
	    shift ; shift ;;
	-l)
	    libs="$libs -l$2"
	    shift ; shift ;;
	-o)
	    exe=$2
	    shift ; shift ;;
	-p)
	    path=$2
	    shift ; shift ;;
	--)
	    shift ; break;;
    esac
done

file=$*


# these are the various programs I need to do the linking.
getsyms="getsyms -p $path -c $channel"
ld=@target_alias@-ld
size=@target_alias@-size
nm=@target_alias@-nm
objcopy=@target_alias@-objcopy

echo "$myself: Reading symbols from $path channel $channel ..." 1>&2
$getsyms symbols > $tmp.syms

# Make up a linker script that I can use to fully link (but not
# relocate) the module. The output from this link should have all its
# symbols resolved and all its contents loaded into the common
# sections.
cat >> $tmp.syms <<EOF
SECTIONS
{
    .text : {
	*(.text)
	*(.text)
	. = ALIGN(4);
	___CTOR_TABLE__ = .;
	*(.ctors)
	LONG(0);
	___DTOR_TABLE__ = .;
	*(.dtors)
	LONG(0);
    }
    .data : {
	*(.data)
    }
    .bss : {
	*(.bss)
    }

}

STARTUP(dlocore.o)

EOF

# Link the object file with the symbols from the target to generate an
# image that is ready to relocate.

echo "$myself: Linking module with target symbols ..." 1>&2
$ld -r $libdirs -o $tmp.o1 -M -T $tmp.syms $file $libs > $tmp.map1

if [ $? != 0 ]
then
    echo "Symbols do not match the target." 1>&2
    exit 1
fi

# Find out how big the loadable module is, and request of the target
# an allocation. The target will send me back the address of some
# memory I can use for the module.
set -- `$size $tmp.o1 | grep $tmp.o1`
size=`expr $4 + 8`

set -- `$getsyms allocate $size`
org=$1
len=$2

# With the allocation returned by the target, re-relocate the module
# into the address range that I've been assigned. Take that result and
# generate a binary image that I can then send to the target.
cat > $tmp.ld <<EOF
MEMORY
{
    place : org = $org, len = $len
}

SECTIONS
{
    .text : { $tmp.o1(.text .data) } > place
    .bss (NOLOAD) :  { $tmp.o1(.bss) }  > place
}
EOF

echo "$myself: Relocating module to suit target ..." 1>&2
$ld -o $exe $tmp.o1 $tmp.ld

if [ $? != 0 ]
then
    echo "Symbols do not match the target." 1>&2
    exit 1
fi

# Get the symbol table from the object module, if it exists.
sym_addr=0
set -- `$nm $exe | fgrep '$'ymbol_table`
if [ $? = 0 ]
then
    echo "$myself: Symbol table at 0x$1 ..." 1>&2
    sym_addr="0x$1"
fi

$objcopy -v -O binary $exe $tmp.bin > /dev/null

echo "$myself: Loading module into $path channel $channel ..." 1>&2
$getsyms -S $sym_addr image $org $tmp.bin

rm -f $tmp.*

echo "$myself: done" 1>&2
