#ifndef __coff_H
#define __coff_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 */
#ident "$Id: coff.h,v 1.3 1997/12/18 22:53:54 steve Exp $"
/*
 * $Log: coff.h,v $
 * Revision 1.3  1997/12/18 22:53:54  steve
 *  properly pack layout structures.
 *
 * Revision 1.2  1996/10/28 03:46:50  steve
 *  License notice.
 *
 * Revision 1.1  1996/10/08 03:28:52  steve
 *  Add the ISE bootstrap prom.
 *
 */

/*
 * NOTE: All numbers are stored in a coff header little-endian. There
 * is no guaranteed alignment, so store things as character and
 * construct the numbers manually.
 */

/*
 * The COFF file starts with a file_header. This gives me a
 * description of the structure of the reset of the file. The file
 * starts at location 0 with this header. Then, the optional header.
 */
struct file_header {
      unsigned char f_magic[2];	/* magic number			*/
      unsigned char f_nscns[2];	/* number of sections		*/
      unsigned char f_timdat[4];	/* time & date stamp		*/
      unsigned char f_symptr[4];	/* file pointer to symtab	*/
      unsigned char f_nsyms[4];	/* number of symtab entries	*/
      unsigned char f_opthdr[2];	/* sizeof(optional hdr)		*/
      unsigned char f_flags[2];	/* flags			*/
} __attribute__((packed));

/*
 * Right after the optional header, find section headers for each
 * section. All the section headers are together in the COFF file. The
 * data contained in the sections is located by following the scnptr,
 * which is relative the beginning of the file.
 */
struct section_header {
               char s_name[8];	/* section name			*/
      unsigned char s_paddr[4];	/* physical address, aliased s_nlib */
      unsigned char s_vaddr[4];	/* virtual address		*/
      unsigned char s_size[4];	/* section size			*/
      unsigned char s_scnptr[4];	/* file ptr to raw data for section */
      unsigned char s_relptr[4];	/* file ptr to relocation	*/
      unsigned char s_lnnoptr[4];	/* file ptr to line numbers	*/
      unsigned char s_nreloc[2];	/* number of relocation entries	*/
      unsigned char s_nlnno[2];	/* number of line number entries*/
      unsigned char s_flags[4];	/* flags			*/
      unsigned char pad[4];
} __attribute__((packed));

#endif
