
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *   Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 *
 * $Id: linux.ld,v 1.2 1996/11/10 23:35:59 steve Exp $
 *
 * $Log: linux.ld,v $
 * Revision 1.2  1996/11/10 23:35:59  steve
 *  Fix malloc into .bss.
 *
 * Revision 1.1  1996/11/08 21:40:52  steve
 *  Divide up thread_create for easier porting, Add the
 *  sparc_linux and i386_linux targets, and adjust makefiles
 *  to allow for them. Document the new concept of virtual boards.
 *
 *  Change the linker output file extension to .exe, because
 *  some targets use ELF and not COFF.
 *
 *  Get rid of some trivial warnings.
 *
 */

ENTRY(start)
STARTUP(locore.o)

SECTIONS
{
	. = . + 0x10000;

	  /* Executable code, constants, and the constructor table
	     go in the beginning of the memory. */

	.text : {
		*(.text)
		. = ALIGN(4);
		__CTOR_TABLE__ = .;
		*(.ctors)
		LONG(0);
		__DTOR_TABLE__ = .;
		*(.dtors)
		LONG(0);
	} =0 


	  /* Initialized data goes into the DRAM, right afer the
	     .text section. Note that I take special care that the
	     _etext and _bdata symbols are adjacent. This causes
	     the startup code to NOT copy the .data section on
	     when the program starts. */

	.data : {
		. = ALIGN(16); etext = .; bdata = .;
		*(.data)
		*(COMMON)
		edata = .;
	}


	  /* The .bss section is uninitialized data. I need to link
	     to it, but it doesn't get loaded. */

	.bss (NOLOAD) : {
		*(.bss)
		end = ALIGN(8);
		. = . + 4M;
		end_heap = .;
	}

	 /* These are just the way Linux/SPARC sets up the main stack. */
	estack = 0xf0000000;
	bstack = estack - 1M;
}
